@@ -31,6 +31,7 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "cil_fqn.h"
 #include "cil_internal.h"
 #include "cil_log.h"
 #include "cil_strpool.h"
@@ -103,7 +104,7 @@ static int __cil_fqn_qualify_blocks(__attribute__((unused)) hashtab_key_t k, has
 		case CIL_SYM_IPADDRS:
 		case CIL_SYM_NAMES:
 		case CIL_SYM_PERMX:
-			/* These do not show up in the kernal policy */
+			/* These do not show up in the kernel policy */
 			break;
 		case CIL_SYM_POLICYCAPS:
 			/* Valid policy capability names are defined in libsepol */
@@ -121,7 +122,7 @@ static int __cil_fqn_qualify_blocks(__attribute__((unused)) hashtab_key_t k, has
 
 exit:
 	if (rc != SEPOL_OK) {
-		cil_log(CIL_ERR,"Problem qualifying names in block at line %d of %s\n", child_args.node->line, child_args.node->path);
+		cil_tree_log(node, CIL_ERR,"Problem qualifying names in block");
 	}
 
 	return rc;
